@@ -11,7 +11,7 @@
 (*  library, see the COPYING file for more information.                   *)
 (**************************************************************************)
 
-open ExtLib
+open! ExtLib
 open Common
 open Debian
 open Algo
@@ -112,7 +112,7 @@ let string_of_list string_of_item sep l =
   Buffer.contents buf
 ;;
 
-let string_of_pkglist = string_of_list string_of_package ", ";;
+let string_of_pkglist = string_of_list (string_of_package ~noversion:false) ", ";;
 
 (* check if a package is member of a package list *)
 let pkg_list_mem l pkg =
@@ -167,9 +167,7 @@ let debessential_of_cudfpkg pkg =
     failwith "can only get essential property of binary packages";
   let ess = try
       CudfAdd.decode (Cudf.lookup_package_property pkg "essential")
-    with Not_found ->
-      failwith (Printf.sprintf "cudf package %s does not have property essential"
-                  (CudfAdd.string_of_package pkg))
+    with Not_found -> "false"
   in
   match ess with
   | "true" -> true
@@ -257,34 +255,26 @@ let unique ll =
   ) CudfSet.empty ll)
 ;;
 
-let optimal_subset ?(global_constraints=true) ?(available=(fun _ -> true)) pkg univ closure =
-  let globalid = Cudf.universe_size univ in
-  (* if global constraints are to be enforced, then all essential packages are
-   * added to the installation request *)
-  let essential =
-    if global_constraints then
-      List.map
-        (fun pkg -> (pkg.Cudf.package, Some (`Eq, pkg.Cudf.version)))
-        (CudfAdd.latest (Cudf.get_packages
-           ?filter:(Some (fun pkg -> pkg.Cudf.keep = `Keep_package ))
-           univ))
-    else
-      []
+let optimal_subset ?(global_constraints=[]) ?(available=(fun _ -> true)) pkg univ closure =
+  let dummy = { Algo.Depsolver.dummy_request with
+                Cudf.depends =
+                  List.map (fun (_,pkglist) ->
+                      List.map (fun uid ->
+                          let pkg = CudfAdd.inttopkg univ uid in
+                          (pkg.Cudf.package,Some(`Eq,pkg.Cudf.version))
+                        ) pkglist
+                    ) global_constraints }
   in
-  (* removes the globalid from the closure (if any) and adds the property
-   * "notavailable" to all binary packages that satisfy the predicate function
-   * "available" *)
   let cudf_closure = List.filter_map (fun i ->
-      if i = globalid then
-        None
+      let pkg = try
+          CudfAdd.inttopkg univ i
+        with Not_found -> fatal "Cannot find pkg for int %d" i
+      in
+      if List.mem ("type", `String "src") pkg.Cudf.pkg_extra then
+        Some pkg
       else begin
-        let pkg = CudfAdd.inttovar univ i in
-        if List.mem ("type", `String "src") pkg.Cudf.pkg_extra then
-          Some pkg
-        else begin
-          let notavail = if available pkg then `Int 0 else `Int 1 in
-          Some { pkg with Cudf.pkg_extra = ("notavailable", notavail) :: pkg.Cudf.pkg_extra }
-        end
+        let notavail = if available pkg then `Int 0 else `Int 1 in
+        Some { pkg with Cudf.pkg_extra = ("notavailable", notavail) :: pkg.Cudf.pkg_extra }
       end
     ) closure
   in
@@ -297,15 +287,15 @@ let optimal_subset ?(global_constraints=true) ?(available=(fun _ -> true)) pkg u
   let newuniverse = Cudf.load_universe cudf_closure in
   let preamble = Debcudf.preamble in
   let preamble = CudfAdd.add_properties preamble [("notavailable",(`Int (Some 0)))] in
-  let request_pkgname = pkg.Cudf.package in
+  let install = (pkg.Cudf.package, Some (`Eq, pkg.Cudf.version)) in
   let request = { Cudf.default_request
                   with Cudf.request_id = "";
-                       Cudf.install = (request_pkgname, Some (`Eq, pkg.Cudf.version)) :: essential }
+                       Cudf.install = [install] }
   in
   let criteria = "-sum(solution,notavailable),-count(solution)" in
   let cmd = "aspcud $in $out $pref" in
   try
-    Depsolver.check_request ~cmd ~criteria (preamble,newuniverse,request)
+    Depsolver.check_request ~dummy ~cmd ~criteria (preamble,newuniverse,request)
   with e ->
     warning "exception when handling %s" pkg.Cudf.package;
     raise e
@@ -324,7 +314,7 @@ let optimal_subset ?(global_constraints=true) ?(available=(fun _ -> true)) pkg u
  *
  *
  *)
-let partition_deps ?(partition_optimizer=(fun p s -> s)) pool univ iss pkg =
+let partition_deps ?(partition_optimizer=(fun _ s -> s)) pool univ iss pkg =
   let to_set l = List.fold_right IntSet.add l IntSet.empty in
   let globalid = Cudf.universe_size univ in
   let l = List.map (fun vpkglist ->
@@ -335,12 +325,11 @@ let partition_deps ?(partition_optimizer=(fun p s -> s)) pool univ iss pkg =
       debug "More then one package in the intersection";
     if not(IntSet.is_empty intrs) then begin
       let pid = IntSet.choose intrs in
-      let dc = Depsolver_int.dependency_closure_cache pool [pid; globalid] in
-      let dcs = to_set dc in
+      let dc = Depsolver_int.dependency_closure_cache pool [pid] in
+      (* the closure contains the globalid which we do not want *)
+      let dcs = IntSet.remove globalid (to_set dc) in
       (* calculate the intersection between the chosen installation set and the
        * dependency closure of pid *)
-      (* the result will not include globalid which might've been part of "dc"
-       * because "iss" doesn't contain it *)
       (* but the result will include "pid". This is important because the
        * buildGraph will make connections to their source packages of all
        * binary packages in the IS but not from the binary package the IS
@@ -355,10 +344,15 @@ let partition_deps ?(partition_optimizer=(fun p s -> s)) pool univ iss pkg =
   l
 ;;
 
-let compute_dependency_sets_opt ?(global_constraints=true) ?(partition=true) ?(available=(fun _ -> true)) opt_partition_cache pool univ srcpkg =
-  let id = CudfAdd.vartoint univ srcpkg in
+let compute_dependency_sets_opt ?(global_constraints=[]) ?(partition=true) ?(available=(fun _ -> true)) opt_partition_cache pool univ srcpkg =
+  let id = CudfAdd.pkgtoint univ srcpkg in
   let globalid = Cudf.universe_size univ in
-  let closure = Depsolver_int.dependency_closure_cache pool [id; globalid] in
+  (* remove the global id from the dependency closure *)
+  let closure = List.filter_map (function
+      |i when i = globalid -> None
+      |i -> Some i)
+      (Depsolver_int.dependency_closure_cache pool [id])
+  in
   (* given a package and an installation set, passes that set to a solver to
    * find a smaller set
    *
@@ -370,13 +364,13 @@ let compute_dependency_sets_opt ?(global_constraints=true) ?(partition=true) ?(a
     match Hashtbl.find_option opt_partition_cache cache_key with
     | Some is -> is
     | None -> begin
-        let r = optimal_subset ~available (CudfAdd.inttovar univ pid) univ (IntSet.elements dcs) in
+        let r = optimal_subset ~global_constraints ~available (CudfAdd.inttopkg univ pid) univ (IntSet.elements dcs) in
         let is = match r with
           |Algo.Depsolver.Error s -> fatal "%s" s;
           |Algo.Depsolver.Unsat _ -> fatal "this must not happen";
           |Algo.Depsolver.Sat (_,soluniv) ->
             Cudf.fold_packages (fun acc pkg ->
-                IntSet.add (CudfAdd.vartoint univ pkg) acc
+                IntSet.add (CudfAdd.pkgtoint univ pkg) acc
               ) IntSet.empty soluniv
         in
         Hashtbl.add opt_partition_cache cache_key is;
@@ -402,7 +396,7 @@ let compute_dependency_sets_opt ?(global_constraints=true) ?(partition=true) ?(a
     |Algo.Depsolver.Sat (_,soluniv) -> begin
         (* remove source package from installation set *)
         let iss = Cudf.fold_packages (fun acc pkg ->
-            IntSet.add (CudfAdd.vartoint univ pkg) acc
+            IntSet.add (CudfAdd.pkgtoint univ pkg) acc
           ) IntSet.empty soluniv
         in
         if partition then
@@ -414,23 +408,21 @@ let compute_dependency_sets_opt ?(global_constraints=true) ?(partition=true) ?(a
 ;;
 
 (* compute_dependency_sets using low level integer interface *)
-let compute_dependency_sets ?(global_constraints=true) ?(partition=true) custom_is_ht pool univ srcpkg =
-  let id = CudfAdd.vartoint univ srcpkg in
-  (* globalid is a fake package indentifier used to encode global
-   * constraints in the universe *)
-  let globalid = Cudf.universe_size univ in
-  let closure = Depsolver_int.dependency_closure_cache pool [id; globalid] in
-  let solver = Depsolver_int.init_solver_closure pool closure in
-  let req = if global_constraints then begin (Some globalid,[id]) end else (None,[id]) in
+let compute_dependency_sets ?(global_constraints=[]) ?(partition=true) custom_is_ht pool univ srcpkg =
+  let id = CudfAdd.pkgtoint univ srcpkg in
+  let closure = Depsolver_int.dependency_closure_cache pool [id] in
+  let solver = Depsolver_int.init_solver_closure ~global_constraints pool closure in
+  let req = [id] in
   let excludeset = Hashtbl.find_option custom_is_ht srcpkg.Cudf.package in
+  let explain = true in
   let d = match excludeset with
     | Some es -> begin
         (* generate an installation set without one or more packages *)
         (* get ids to not include *)
         let excludelits = List.filter_map (fun pid ->
-          if pid = id || pid = globalid then None
+          if pid = id then None
           else begin
-            let pkg = CudfAdd.inttovar univ pid in
+            let pkg = CudfAdd.inttopkg univ pid in
             if StringSet.mem pkg.Cudf.package es then
               Some (Depsolver_int.S.lit_of_var (solver.Depsolver_int.map#vartoint pid) false)
             else None
@@ -439,23 +431,26 @@ let compute_dependency_sets ?(global_constraints=true) ?(partition=true) custom_
         match excludelits with
           | [] -> begin (* empty list. Solve normally *)
               warning "list of packages to exclude from the IS of %s is empty" (srcpkg.Cudf.package);
-              Depsolver_int.solve solver req
+              Depsolver_int.solve ?tested:None ~explain solver req
             end
           | _ -> begin
               let solver = Depsolver_int.copy_solver solver in
               Depsolver_int.S.add_rule solver.Depsolver_int.constraints (Array.of_list excludelits) [];
-              Depsolver_int.solve solver req
+              Depsolver_int.solve ?tested:None ~explain solver req
             end
       end
     | None -> (* generate an installation set normally *)
-        Depsolver_int.solve solver req
+        Depsolver_int.solve ?tested:None ~explain solver req
   in
   match d with
-  |Diagnostic_int.Success f_int -> begin
-    (* remove source package and globalid from installation set *)
-    let iss = List.fold_left (fun s i ->
-      if i = globalid || i = id then s else IntSet.add i s
-    ) IntSet.empty (f_int ()) in
+  |Diagnostic.SuccessInt f_int -> begin
+    let globalid = solver.Depsolver_int.map#vartoint (Cudf.universe_size univ) in
+    let cudfis = List.filter_map (function
+            |i when i = globalid -> None
+            |i -> Some (solver.Depsolver_int.map#inttovar i))
+        (f_int ())
+    in
+    let iss = List.fold_right IntSet.add cudfis IntSet.empty in
     if partition then
       iss, (partition_deps pool univ iss srcpkg)
     else
@@ -464,7 +459,7 @@ let compute_dependency_sets ?(global_constraints=true) ?(partition=true) custom_
   | _ -> begin
       if Util.Debug.is_enabled "BootstrapCommon" then begin
         (*let result = Depsolver.diagnosis solver.Depsolver_int.map univ d req in*)
-        Diagnostic.fprintf ~explain:true ~failure:true Format.err_formatter { Diagnostic.result = Depsolver.result solver.Depsolver_int.map univ d; request = Depsolver.request univ req }
+        Diagnostic.fprintf ~explain:true ~failure:true Format.err_formatter { Diagnostic.result = Diagnostic.result solver.Depsolver_int.map univ d; request = Diagnostic.request univ req }
       end;
       (* source package could not be compiled. If the installation set was chosen
        * manually, fail. Otherwise just throw a warning. *)
@@ -606,7 +601,7 @@ let read_package_file ?(archs=[]) tocudf f =
       try tocudf pkg
       with Not_found ->
         failwith (Printf.sprintf "cannot find cudf version for %s - do \
-                  your foreground packages contain it?" (pkg.Packages.name));
+                  your foreground packages contain it?" (pkg#name));
     in
     CudfAdd.Cudf_set.add cudfpkg acc
   ) CudfAdd.Cudf_set.empty l
@@ -615,7 +610,7 @@ let read_package_file ?(archs=[]) tocudf f =
 let more_problems_callback printer universe results summary d =
   if summary then Diagnostic.collect results d ;
   match d with
-  |{Diagnostic.result = Diagnostic.Failure (f) ; request = [r] } -> begin
+  |{Diagnostic.result = Diagnostic.Failure (f) } -> begin
       let new_reasons = f () in
       List.iter (fun reason ->
           match reason with
@@ -663,20 +658,18 @@ let more_problems_callback printer universe results summary d =
       printer d
     end
   |{Diagnostic.result = Diagnostic.Success _} -> ()
-  |_ -> failwith "impossible #3"; (* we should only get single packages *)
 ;;
 
 let parse_debian_pkgstring universe native_arch pkgstring =
-  (* FIXME: this will break if cudf encoding changes *)
   let add_name_arch n a = CudfAdd.encode (Printf.sprintf "%s:%s" n a) in
   let parse_vpkglist s =
     let _loc = Format822.dummy_loc in
     List.map (function
         |((n,a),Some("=",v)) -> (n,a,v)
-        |((n,a),None) ->
-          raise (Debian.Packages.ParseError (s,"you must specify a version" ))
-        |_ -> raise (Debian.Packages.ParseError (s,""))
-      ) (Debian.Packages.parse_vpkglist (_loc,s))
+        |(_,None) ->
+          raise (Format822.ParseError ([],s,"you must specify a version" ))
+        |_ -> raise (Format822.ParseError ([],s,""))
+      ) (Pef.Packages.parse_vpkglist ("pkgstring", (_loc,s)))
   in
   List.fold_left (fun acc (n,a,v) ->
       let cudfname = match a with
@@ -697,6 +690,6 @@ let parse_debian_pkgstring universe native_arch pkgstring =
         | [p] -> p
         | _ -> failwith (Printf.sprintf "more than one match for %s" cudfname)
       in
-      IntSet.add (CudfAdd.vartoint universe pkg) acc
+      IntSet.add (CudfAdd.pkgtoint universe pkg) acc
     ) IntSet.empty (parse_vpkglist pkgstring)
 ;;
