@@ -11,7 +11,7 @@
 (*  library, see the COPYING file for more information.                   *)
 (**************************************************************************)
 
-open ExtLib
+open! ExtLib
 open Common
 open Debian
 open Algo
@@ -102,12 +102,42 @@ let graph_helper ?(num_cores=1) available bin2src univ partitionofsrc sl =
         (* generate dependency sets outside of graph generation because the
          * graph is imperative and we must not executed code with side-effects
          * inside a parallel block *)
-        let instsets = Parmap.parmap ~ncores:num_cores (fun srcpkg ->
+        let worker srcpkg =
             Util.Progress.progress progressbar_build_graph;
             let _, partitions = partitionofsrc srcpkg in
-            let srcpkgid = CudfAdd.vartoint univ srcpkg in
+            let srcpkgid = CudfAdd.pkgtoint univ srcpkg in
             (srcpkgid, partitions)
-          ) (Parmap.L todo) in
+        in
+        let instsets = if num_cores > 1 then
+            (* On 32 bit architectures it can happen that the result returned
+             * by parmap is too big to fit into the OCaml constraints for
+             * maximum allocatable memory. Should this problem occur, we
+             * repeat the computation with smaller chunk sizes.
+             * Unfortunately, should the result be too big and the child
+             * process throws an Out_of_memory error, then all the parent
+             * process sees is the Failure exception that the marshalling
+             * module throws. Thus, the Failure we do catch might also be
+             * caused by other problems than an Out_of_memory exception.
+             * https://github.com/rdicosmo/parmap/issues/46
+             *)
+            let rec splitjob maxsplits todo =
+              try
+                Parmap.parmap ~ncores:num_cores worker (Parmap.L todo)
+              with
+              | Failure "input_value_from_block: bad object" -> begin
+                  if maxsplits <= 0 then fatal "exceeded maximum split depth";
+                  warning "parmap raised out of memory exception with list length %d, splitting job..." (List.length todo);
+                  let l1, l2 = List.split_nth ((List.length todo)/2) todo in
+                  let res1 = splitjob (maxsplits-1) l1 in
+                  let res2 = splitjob (maxsplits-1) l2 in
+                  List.append res1 res2
+                end
+            in
+            let maxsplits = 4 in
+            splitjob maxsplits todo
+          else
+            List.map worker todo
+        in
         (* we must not do this iteration in the block above because side
          * effects from parallel blocks are discarded *)
         List.iter (fun (srcpkgid,_) -> Hashtbl.add seen srcpkgid ()) instsets;
@@ -136,10 +166,13 @@ let graph_helper ?(num_cores=1) available bin2src univ partitionofsrc sl =
                  * This is important to not include non-strong direct
                  * dependencies in a strong dependency graph. *)
                 IntSet.fold (fun pid acc ->
-                    let binpkg = CudfAdd.inttovar univ pid in
+                    let binpkg = try
+                        CudfAdd.inttopkg univ pid
+                      with Not_found -> fatal "cannot find pkg for pid %d" pid
+                    in
                     if not (available binpkg) then begin
                       let srcpkgdep = bin2src binpkg in
-                      let srcpkgdepid = CudfAdd.vartoint univ srcpkgdep in
+                      let srcpkgdepid = CudfAdd.pkgtoint univ srcpkgdep in
                       let dst = Unique.create (SrcPkg srcpkgdepid) in
                       begin try begin match G.E.label (G.find_edge g instset dst) with
                         |{ depend = BuildsFrom s } -> s := IntSet.add pid !s
@@ -175,10 +208,16 @@ let get_src_package ?(allowmismatch=false) univ pkg = try
 ;;
 
 (* create and time a normal buildgraph by calculating installation sets *)
-let dist_graph ?(global_constraints=true) ?(available=(fun _ -> true)) ?(allowmismatch=false) ?(opt=false) ?(num_cores=1) custom_is_ht univ sl =
+let dist_graph ?(global_constraints=[]) ?(available=(fun _ -> true)) ?(allowmismatch=false) ?(opt=false) ?(num_cores=1) custom_is_ht univ sl =
   Util.Timer.start timer_build_graph;
   Util.Progress.set_total progressbar_build_graph (List.length sl);
 
+  let global_constraints =
+    List.map (fun (vpkg,pkglist) ->
+	(vpkg,List.map (CudfAdd.pkgtoint univ) pkglist)
+    ) global_constraints
+  in
+
   let bin2src = get_src_package ~allowmismatch univ in
   let pool = Depsolver_int.init_pool_univ ~global_constraints univ in
 
@@ -193,23 +232,28 @@ let dist_graph ?(global_constraints=true) ?(available=(fun _ -> true)) ?(allowmi
 ;;
 
 (* create and time a buildgraph by calculating installation closures *)
-let closure_graph ?(global_constraints=true) ?(available=(fun _ -> true)) ?(allowmismatch=false) ?(num_cores=1) univ sl =
+let closure_graph ?(global_constraints=[]) ?(available=(fun _ -> true)) ?(allowmismatch=false) ?(num_cores=1) univ sl =
   Util.Timer.start timer_build_graph;
   Util.Progress.set_total progressbar_build_graph (List.length sl);
+  let global_constraints =
+    List.map (fun (vpkg,pkglist) ->
+	(vpkg,List.map (CudfAdd.pkgtoint univ) pkglist)
+    ) global_constraints
+  in
 
   let bin2src = get_src_package ~allowmismatch univ in
   let pool = Depsolver_int.init_pool_univ ~global_constraints univ in
-  let globalid = Cudf.universe_size univ in
 
+  let globalid = Cudf.universe_size univ in
   let to_set l = List.fold_right IntSet.add l IntSet.empty in
+
   (* rule: connect everything to everything else *)
   let partitionofsrc src =
     let l = List.fold_left (fun acc vpkglist ->
       let disj = CudfAdd.resolve_vpkgs_int univ vpkglist in
       List.fold_left (fun acc2 pid ->
-          let dc = Depsolver_int.dependency_closure_cache pool [pid; globalid] in
-          let dcs = IntSet.remove globalid (to_set dc) in
-          (pid, dcs)::acc2
+          let dcs = Depsolver_int.dependency_closure_cache pool [pid] in
+          (pid, IntSet.remove globalid (to_set dcs))::acc2
       ) acc disj
     ) [] src.Cudf.depends in
     IntSet.empty, l
@@ -219,17 +263,21 @@ let closure_graph ?(global_constraints=true) ?(available=(fun _ -> true)) ?(allo
 ;;
 
 (* create and time a normal buildgraph by calculating strong dependencies *)
-let strong_graph ?(global_constraints=true) ?(available=(fun _ -> true)) ?(allowmismatch=false) ?(num_cores=1) univ sl =
+let strong_graph ?(global_constraints=[]) ?(available=(fun _ -> true)) ?(allowmismatch=false) ?(num_cores=1) univ sl =
   Util.Timer.start timer_build_graph;
   Util.Progress.set_total progressbar_build_graph (List.length sl);
+  let global_constraints =
+    List.map (fun (vpkg,pkglist) ->
+	(vpkg,List.map (CudfAdd.pkgtoint univ) pkglist)
+    ) global_constraints
+  in
 
   let bin2src = get_src_package ~allowmismatch univ in
   let pool = Depsolver_int.init_pool_univ ~global_constraints univ in
-  let globalid = Cudf.universe_size univ in
 
   let sdg = Strongdeps.strongdeps_univ univ in
   (* connect all binary packages to the Essential:yes packages *)
-  if global_constraints then begin
+  if List.length global_constraints > 0 then begin
     (* get all binary packages that are Essential:yes *)
     let essential = Cudf.get_packages
         ~filter:(fun pkg ->
@@ -251,7 +299,7 @@ let strong_graph ?(global_constraints=true) ?(available=(fun _ -> true)) ?(allow
           essential) sdg
   end;
 
-  let tointset l = List.fold_left (fun acc e -> IntSet.add (CudfAdd.vartoint univ e) acc) IntSet.empty l in
+  let tointset l = List.fold_left (fun acc e -> IntSet.add (CudfAdd.pkgtoint univ e) acc) IntSet.empty l in
   let to_set l = List.fold_right IntSet.add l IntSet.empty in
   (* rule: connect each source package to those source packages which build the
    * binary packages that make their strong dependencies *)
@@ -266,7 +314,7 @@ let strong_graph ?(global_constraints=true) ?(available=(fun _ -> true)) ?(allow
       let l = List.fold_left (fun acc vpkglist ->
           let disj = CudfAdd.resolve_vpkgs_int univ vpkglist in
           List.fold_left (fun acc2 pid ->
-              let dc = Depsolver_int.dependency_closure_cache pool [pid; globalid] in
+              let dc = Depsolver_int.dependency_closure_cache pool [pid] in
               let dcs = to_set dc in
               let inters = IntSet.inter strongdepss dcs in
               if not(IntSet.is_empty inters) then (pid, inters)::acc2 else acc2
@@ -283,7 +331,7 @@ let strong_graph ?(global_constraints=true) ?(available=(fun _ -> true)) ?(allow
 ;;
 
 (* Build the dependency source graph bfs from a root node *)
-let dist_graph_bfs ?(global_constraints=true) ?(maxdepth=max_int) ?(available=(fun _ -> true)) ?(allowmismatch=false) custom_is_ht univ rootlist =
+let dist_graph_bfs ?(global_constraints=[]) ?(maxdepth=max_int) ?(available=(fun _ -> true)) ?(allowmismatch=false) custom_is_ht univ rootlist =
   Util.Timer.start timer_build_graph_bfs;
   Util.Progress.set_total progressbar_build_graph_bfs (Cudf.universe_size univ);
 
@@ -299,7 +347,7 @@ let dist_graph_bfs ?(global_constraints=true) ?(maxdepth=max_int) ?(available=(f
 
   begin try while (Queue.length queue > 0) do
     let (srcpkg,level) = Queue.take queue in
-    let src = Unique.create (SrcPkg (CudfAdd.vartoint univ srcpkg)) in
+    let src = Unique.create (SrcPkg (CudfAdd.pkgtoint univ srcpkg)) in
     G.add_vertex g src;
     if not(Hashtbl.mem visited src) && level < maxdepth then begin
       Util.Progress.progress progressbar_build_graph_bfs;
@@ -307,17 +355,17 @@ let dist_graph_bfs ?(global_constraints=true) ?(maxdepth=max_int) ?(available=(f
       (* if the package cannot be compiled the List.iter will not happen *)
       let _, partitions = BootstrapCommon.compute_dependency_sets ~global_constraints ~partition:true custom_is_ht pool univ srcpkg in
       List.iter (fun (pid,is) ->
-          (* let open Cudf in Cudf_printer.pp_package stdout (CudfAdd.inttovar univ pid); *)
+          (* let open Cudf in Cudf_printer.pp_package stdout (CudfAdd.inttopkg univ pid); *)
           let instset = Unique.create (InstSet (pid,is)) in
           let label = { depend = BuildDep; annotation = [] } in
           let edge = (src,label,instset) in
           (* src -> instset edge *)
           G.add_edge_e g edge;
           IntSet.iter (fun pid ->
-            let binpkg = CudfAdd.inttovar univ pid in
+            let binpkg = CudfAdd.inttopkg univ pid in
             if not (available binpkg) then begin
               let srcpkgdep = bin2src binpkg in
-              let dst = Unique.create (SrcPkg (CudfAdd.vartoint univ srcpkgdep)) in
+              let dst = Unique.create (SrcPkg (CudfAdd.pkgtoint univ srcpkgdep)) in
               (* we add a new src to the queue of sources to compile *)
               Queue.add (srcpkgdep,level+1) queue;
               (* the edge (instset -> src edge) aleady exists. In this case we
@@ -366,7 +414,7 @@ let from_ic universe native_arch ic =
            failwith (Printf.sprintf "cannot find cudf package %s (= %d)"
                        cudfname cudfversion)
        in
-       Unique.create (SrcPkg (CudfAdd.vartoint universe srcpkg))
+       Unique.create (SrcPkg (CudfAdd.pkgtoint universe srcpkg))
      end
      | "InstSet" -> begin
        let cudfname = CudfAdd.encode (getstr "cudfname" l) in
@@ -377,7 +425,7 @@ let from_ic universe native_arch ic =
            failwith (Printf.sprintf "cannot find cudf package %s (= %d)"
                        cudfname cudfversion)
        in
-       let pid = CudfAdd.vartoint universe pkg in
+       let pid = CudfAdd.pkgtoint universe pkg in
        let binaries = try getstr "binaries" l
          with Not_found -> failwith "cannot find mandatory InstSet vertex attribute \"binaries\""
        in
@@ -440,12 +488,7 @@ module Graphml (U : sig val univ : Cudf.universe end)  = struct
        "cudfversion","int",None;
        "cudfname","string",None;
        "architecture","string",None;
-       "priority","string",None;
        "type","string",None;
-       "essential","string",None;
-       "buildessential","string",None;
-       "source","string",None;
-       "sourcenumber","string",None;
        "kind","string",None;
       ]
 
@@ -464,7 +507,7 @@ module Graphml (U : sig val univ : Cudf.universe end)  = struct
     let vertex = Unique.value vertex in
     match vertex with
     |SrcPkg id->
-      let pkg = CudfAdd.inttovar U.univ id in
+      let pkg = CudfAdd.inttopkg U.univ id in
       let kind = ("kind",string_of_vertex_kind vertex) in
       let cudfname = ("cudfname", CudfAdd.decode pkg.Cudf.package) in
       let prop =
@@ -492,13 +535,13 @@ module Graphml (U : sig val univ : Cudf.universe end)  = struct
       let binaries =
         let bl =
           List.map (fun pid ->
-            let pkg = CudfAdd.inttovar U.univ pid in
+            let pkg = CudfAdd.inttopkg U.univ pid in
             BootstrapCommon.string_of_package pkg
           ) (IntSet.elements is)
         in
         ("binaries",String.concat "," bl)
       in
-      let pkg = CudfAdd.inttovar U.univ pid in
+      let pkg = CudfAdd.inttopkg U.univ pid in
       let kind = ("kind",string_of_vertex_kind vertex) in
       let cudfname = ("cudfname", CudfAdd.decode pkg.Cudf.package) in
       let prop =
@@ -538,7 +581,7 @@ module Graphml (U : sig val univ : Cudf.universe end)  = struct
         let witness =
           let sl =
             List.map (fun pid ->
-              let pkg = CudfAdd.inttovar U.univ pid in
+              let pkg = CudfAdd.inttopkg U.univ pid in
               BootstrapCommon.string_of_package pkg
             ) (IntSet.elements s)
           in
