@@ -11,7 +11,7 @@
 (*  library, see the COPYING file for more information.                   *)
 (**************************************************************************)
 
-open ExtLib
+open! ExtLib
 open Common
 
 module StringSet = BootstrapCommon.StringSet
@@ -45,7 +45,7 @@ module Make (U : sig val univ : Cudf.universe end) = struct
       let vertex = BuildGraph.Unique.value v in
       match vertex with
         | BuildGraph.SrcPkg id ->
-            let p = CudfAdd.inttovar U.univ id in
+            let p = CudfAdd.inttopkg U.univ id in
             (p, BGE.pkglist_of_vlist (G.succ g v))::acc
         | _ -> acc
     ) g [] in
@@ -69,7 +69,7 @@ module Make (U : sig val univ : Cudf.universe end) = struct
       let vertex = BuildGraph.Unique.value v in
       match vertex with
         | BuildGraph.SrcPkg id ->
-            let p = CudfAdd.inttovar U.univ id in
+            let p = CudfAdd.inttopkg U.univ id in
             let num_builddeps = List.length (G.succ g v) in
             let neededby = G.pred g v in
             let builddepof = List.fold_left (fun acc s -> (G.pred g s)@acc) [] neededby in
@@ -89,7 +89,7 @@ module Make (U : sig val univ : Cudf.universe end) = struct
       let vertex = BuildGraph.Unique.value v in
       match vertex with
         | BuildGraph.InstSet (id,_) ->
-            let p = CudfAdd.inttovar U.univ id in
+            let p = CudfAdd.inttopkg U.univ id in
             let num_srcpkgs = List.length (G.succ g v) in
             let builddepof = G.pred g v in
             (p, num_srcpkgs, BGE.pkglist_of_vlist builddepof)::acc
@@ -108,11 +108,11 @@ module Make (U : sig val univ : Cudf.universe end) = struct
       let vertex = BuildGraph.Unique.value v in
       match vertex with
         | BuildGraph.SrcPkg id ->
-            let p = CudfAdd.inttovar U.univ id in
+            let p = CudfAdd.inttopkg U.univ id in
             let bd = G.fold_succ (fun v2 acc ->
               let vertex2 = BuildGraph.Unique.value v2 in
               match vertex2 with
-                | BuildGraph.InstSet (id,_) -> (CudfAdd.inttovar U.univ id)::acc
+                | BuildGraph.InstSet (id,_) -> (CudfAdd.inttopkg U.univ id)::acc
                 | _ -> failwith "impossible"
             ) g v [] in
             let bds = List.fold_left (fun acc p ->
