@@ -13,7 +13,7 @@
 
 module OCAMLHashtbl = Hashtbl
 
-open ExtLib
+open! ExtLib
 open Common
 open Algo
 
