@@ -55,7 +55,7 @@ module Options = struct
   StdOptions.OutputOptions.add_options ~default options;;
 
   include StdOptions.DistribOptions;;
-  let default = List.filter (fun e -> not (List.mem e ["deb-drop-b-d-indep"; "deb-profiles"; "deb-ignore-essential"; "deb-builds-from"])) StdOptions.DistribOptions.default_options in
+  let default = List.filter (fun e -> not (List.mem e ["deb-drop-b-d-indep"; "deb-drop-b-d-arch"; "deb-profiles"; "deb-ignore-essential"; "deb-builds-from"])) StdOptions.DistribOptions.default_options in
   StdOptions.DistribOptions.add_debian_options ~default options;;
 end
 
