@@ -2,18 +2,22 @@
 
 from __future__ import print_function
 import sys
-sys.path.append('/usr/share/botch')
+
+sys.path.append("/usr/share/botch")
 from util import read_reduced_deps
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     import argparse
+
     parser = argparse.ArgumentParser(
-        description=("Display the difference between two files with " +
-                     "droppable build dependencies"))
-    parser.add_argument('droppable1', type=read_reduced_deps, help='file 1')
-    parser.add_argument('droppable2', type=read_reduced_deps, help='file 2')
-    parser.add_argument(
-        '-v', '--verbose', action='store_true', help='be verbose')
+        description=(
+            "Display the difference between two files with "
+            + "droppable build dependencies"
+        )
+    )
+    parser.add_argument("droppable1", type=read_reduced_deps, help="file 1")
+    parser.add_argument("droppable2", type=read_reduced_deps, help="file 2")
+    parser.add_argument("-v", "--verbose", action="store_true", help="be verbose")
     args = parser.parse_args()
 
     only_in_1 = set(args.droppable1.keys()) - set(args.droppable2.keys())
